// Import necessary modules from Minecraft server API
import { world } from '@minecraft/server';

// Define a function to handle sapling growth
function handleSaplingGrowth(eventData, saplingType, treeStructures, growthChance, growthRate, offsets) {
    // Register a custom component for the sapling
    eventData.blockComponentRegistry.registerCustomComponent(`biomes_and_beyond:${saplingType}_sapling`, {
        // Define the behavior when a player interacts with the block
        onPlayerInteract(e) {
            // Destructure event data for easier access
            const { block, player } = e;

            // Get the block's current location
            const { x, y, z } = block.location;

            // Get the equipment component for the player
            const equipment = player.getComponent('equippable');

            // Get the selected item from the player's mainhand
            const selectedItem = equipment.getEquipment('Mainhand');

            // Check if the selected item is 'minecraft:bone_meal'
            if (selectedItem && (selectedItem.typeId === 'minecraft:bone_meal')) {
                // Define the location for the particle and structure
                const location = { x: x + 0.5, y: y + 0.5, z: z + 0.5 };

                // Spawn 'minecraft:crop_growth_emitter' particle at the location
                block.dimension.spawnParticle('minecraft:crop_growth_emitter', location);

                // Play bonemeal sound at the sapling's location
                player.runCommand(`/playsound item.bone_meal.use @a[r=5] ${x} ${y} ${z}`);

                // Choose a random tree
                const treeIndex = Math.floor(Math.random() * treeStructures.length);
                const structure = treeStructures[treeIndex];
                const offset = offsets[treeIndex];

                // Generate a random number between 0 and 1
                const randomChance = Math.random();

                // Spawn a tree if the random number is less than the growth chance
                if (randomChance < growthChance) {
                    // Execute the /structure command to load the chosen tree at the sapling's location plus the offset
                    block.dimension.runCommand(`/structure load ${structure} ${x + offset.x} ${y + offset.y} ${z + offset.z}`);
                }

                // Reduce bone meal from player's inventory if not in creative mode
                if (player.getGameMode() !== "creative") {
                    if (selectedItem.amount > 1) {
                        selectedItem.amount -= 1;
                        equipment.setEquipment('Mainhand', selectedItem);
                    } else {
                        // Clear the slot if only 1 item left
                        equipment.setEquipment('Mainhand', undefined);
                    }
                }
            }
        },
        // Allows the sapling to grow naturally
        onTick(e) {
            // Destructure event data for easier access
            const { block } = e;

            // Generate a random number between 0 and 1
            const randomChance = Math.random();

            // Get the block's current location
            const { x, y, z } = block.location;

            // Grow a tree if the random number is less than the growth rate
            if (randomChance < growthRate) {
                // Choose a random tree
                const treeIndex = Math.floor(Math.random() * treeStructures.length);
                const structure = treeStructures[treeIndex];
                const offset = offsets[treeIndex];

                // Execute the /structure command to load the chosen tree at the sapling's location plus the offset
                block.dimension.runCommand(`/structure load ${structure} ${x + offset.x} ${y + offset.y} ${z + offset.z}`);
            }
        }
    });
}

// Subscribe to the 'worldInitialize' event to register custom components
world.beforeEvents.worldInitialize.subscribe(eventData => {
    // Define the tree structures for each type of sapling
    const aspenStructures = ['aspen_tree1', 'aspen_tree2', 'aspen_tree3'];
    const mapleStructures = ['maple_tree_1', 'maple_tree_2', 'maple_tree_3', 'maple_tree_4', 'maple_tree_5'];
    const yellowmapleStructures = ['yellow_maple_tree_1', 'yellow_maple_tree_2', 'yellow_maple_tree_3', 'yellow_maple_tree_4', 'yellow_maple_tree_5'];
    const autumnStructures = ['autumn_tree_1', 'autumn_tree_2', 'autumn_tree_3', 'autumn_tree_4', 'autumn_tree_5'];
    const firStructures = ['fir_tree_1', 'fir_tree_2', 'fir_tree_3'];
    const jacarandaStructures = ['jacaranda_tree_1', 'jacaranda_tree_2', 'jacaranda_tree_3'];
    const redwoodStructures = ['redwood_tree_1', 'redwood_tree_2', 'redwood_tree_3'];
    const baobabStructures = ['baobab_tree_1', 'baobab_tree_2', 'baobab_tree_3', 'baobab_tree_4', 'baobab_tree_5', 'baobab_tree_6'];
    const forsakenStructures = ['forsaken_oak_1_a', 'forsaken_oak_1_b', 'forsaken_oak_2_a', 'forsaken_oak_2_b'];
    const chorusStructures = ['chorus_tree_1', 'chorus_tree_2', 'chorus_tree_3'];
    const poppedStructures = ['pink_chorus_tree_1', 'pink_chorus_tree_2', 'pink_chorus_tree_3', 'pink_chorus_tree_4', 'pink_chorus_tree_5', 'pink_chorus_tree_6'];
    const depravedStructures = ['shroomtree_flowered_big_1', 'shroomtree_flowered_big_2', 'shroomtree_flowered_big_3'];
    const coconutStructures = ['palm_tree_1', 'palm_tree_2', 'palm_tree_3'];

    // Define the offsets for each structure type
    const aspenOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }];
    const mapleOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 },  { x: -3, y: 0, z: -5 },  { x: -4, y: 0, z: -4 }];
    const yellowmapleOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 },  { x: -3, y: 0, z: -5 },  { x: -4, y: 0, z: -4 }];
    const autumnOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 },  { x: -3, y: 0, z: -5 },  { x: -4, y: 0, z: -4 }];
    const firOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }];
    const jacarandaOffsets = [{ x: -1, y: 0, z: -3 }, { x: -3, y: 0, z: -5 }, { x: -1, y: 0, z: -3 }];
    const redwoodOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -3, y: 0, z: -3 }];
    const baobabOffsets = [{ x: -4, y: 0, z: -3 }, { x: -4, y: 0, z: -3 }, { x: -4, y: 0, z: -3 }, { x: -4, y: 0, z: -3 }, { x: -4, y: 0, z: -3 }, { x: -4, y: 0, z: -3 }];
    const forsakenOffsets = [{ x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }, { x: -2, y: 0, z: -2 }];
    const chorusOffsets = [{ x: -2, y: 0, z: -3 }, { x: -2, y: 0, z: -4 }, { x: -2, y: 0, z: -7 }];
    const depravedOffsets = [{ x: -4, y: 0, z: -5 }, { x: -4, y: 0, z: -5 }, { x: -4, y: 0, z: -5 },];
    const poppedOffsets = [{ x: -3, y: 0, z: -1 }, { x: -2, y: 0, z: -1 }, { x: -1, y: 0, z: -2 }, { x: -4, y: 0, z: -1 }, { x: -3, y: 0, z: -3 }, { x: -2, y: 0, z: -2 }];
    const coconutOffsets = [{ x: -5, y: 0, z: -4 }, { x: -4, y: 0, z: -4 }, { x: -5, y: 0, z: -7 },];

    // Handle growth for each type of sapling with growth chances using bonemeal (first value), natural growth rates (second value), and offsets
    handleSaplingGrowth(eventData, 'aspen', aspenStructures, 0.35, 0.10, aspenOffsets);
    handleSaplingGrowth(eventData, 'maple', mapleStructures, 0.35, 0.10, mapleOffsets);
    handleSaplingGrowth(eventData, 'yellow_maple', yellowmapleStructures, 0.35, 0.10, yellowmapleOffsets);
    handleSaplingGrowth(eventData, 'autumn', autumnStructures, 0.35, 0.10, autumnOffsets);
    handleSaplingGrowth(eventData, 'fir', firStructures, 0.35, 0.10, firOffsets);
    handleSaplingGrowth(eventData, 'jacaranda', jacarandaStructures, 0.35, 0.10, jacarandaOffsets);
    handleSaplingGrowth(eventData, 'redwood', redwoodStructures, 0.35, 0.10, redwoodOffsets);
    handleSaplingGrowth(eventData, 'baobab', baobabStructures, 0.35, 0.10, baobabOffsets);
    handleSaplingGrowth(eventData, 'forsaken', forsakenStructures, 0.35, 0.10, forsakenOffsets);
    handleSaplingGrowth(eventData, 'chorus', chorusStructures, 0.35, 0.10, chorusOffsets);
    handleSaplingGrowth(eventData, 'popped_chorus', poppedStructures, 0.35, 0.10, poppedOffsets);
    handleSaplingGrowth(eventData, 'depraved', depravedStructures, 0.35, 0.10, depravedOffsets);
    handleSaplingGrowth(eventData, 'coconut', coconutStructures, 0.35, 0.10, coconutOffsets);
});
