import { world, EquipmentSlot, EntityEquippableComponent, GameMode } from '@minecraft/server';

// List of flower types with _top and _bottom blocks
const flowerTypes = [
    { name: "daisy_petals", bottom: "biomes_and_beyond:daisy_petals", top: null, spread: true },
    { name: "bluebell", bottom: "biomes_and_beyond:bluebell", top: null, spread: true },
    { name: "violet_flower", bottom: "biomes_and_beyond:violet_flower", top: null, spread: true },
    { name: "bunchflower", bottom: "biomes_and_beyond:bunchflower", top: null, spread: true },
    { name: "coralroot", bottom: "biomes_and_beyond:coralroot", top: null, spread: true },
    { name: "lavender_flower", bottom: "biomes_and_beyond:lavender_flower", top: null, spread: true },
    { name: "lazarus", bottom: "biomes_and_beyond:lazarus", top: null, spread: true },
    { name: "reeds", bottom: "biomes_and_beyond:reeds", top: null, spread: true },
    { name: "void_blossom", bottom: "biomes_and_beyond:void_blossom", top: "null", spread: false },
    { name: "puffshroom", bottom: "biomes_and_beyond:puffshroom", top: "null", spread: false },
    { name: "mullein", bottom: "biomes_and_beyond:mullein_bottom", top: "biomes_and_beyond:mullein_top", spread: false },
    { name: "cattails", bottom: "biomes_and_beyond:cattail_bottom", top: "biomes_and_beyond:cattail_top", spread: false },
    { name: "jungfern", bottom: "biomes_and_beyond:jungfern_bottom", top: "biomes_and_beyond:jungfern_top", spread: false },
    { name: "void_blossom_bush", bottom: "biomes_and_beyond:void_blossom_bush_bottom", top: "biomes_and_beyond:void_blossom_bush_top", spread: false }
    // Add more flower types here
];

// Function to spawn an item stack anywhere
function spawnItemAnywhere(item, location, dimension) {
    const itemEntity = dimension.spawnItem(item, location);
    return itemEntity;
}

// Function to check and spawn flowers and short grass
function spawnFlowersAndGrassAround(block, dimension, flowerType) {
    const positions = [
        { x: block.location.x + 1, y: block.location.y, z: block.location.z },
        { x: block.location.x - 1, y: block.location.y, z: block.location.z },
        { x: block.location.x, y: block.location.y, z: block.location.z + 1 },
        { x: block.location.x, y: block.location.y, z: block.location.z - 1 },
        { x: block.location.x + 1, y: block.location.y, z: block.location.z + 1 },
        { x: block.location.x + 1, y: block.location.y, z: block.location.z - 1 },
        { x: block.location.x - 1, y: block.location.y, z: block.location.z + 1 },
        { x: block.location.x - 1, y: block.location.y, z: block.location.z - 1 }
    ];

    let hasSpawnedFlower = false;
    let numberOfItemsToSpawn = Math.floor(Math.random() * 3) + 1; // Randomly 1 to 3 items
    let itemsSpawned = 0;

    positions.forEach(pos => {
        if (itemsSpawned >= numberOfItemsToSpawn) return;

        if (Math.random() < 0.3) {
            let targetPos = { ...pos };
            let blockAtPos = dimension.getBlock(targetPos);

            // Adjust the position if the block below is not air
            if (blockAtPos && blockAtPos.typeId !== "minecraft:air") {
                targetPos.y += 1;
                blockAtPos = dimension.getBlock(targetPos);
            }

            // Check if we can place a flower or grass at the adjusted position and the block below is grass or dirt
            const blockBelow = dimension.getBlock({ x: targetPos.x, y: targetPos.y - 1, z: targetPos.z });
            if (blockAtPos && blockAtPos.typeId === "minecraft:air" && 
                blockBelow && (blockBelow.typeId === "minecraft:grass_block" || blockBelow.typeId === "minecraft:dirt")) {
                
                const itemType = Math.random() < 0.5 ? flowerType : "minecraft:tallgrass";
                dimension.runCommand(`setblock ${targetPos.x} ${targetPos.y} ${targetPos.z} ${itemType}`);
                itemsSpawned += 1;
                hasSpawnedFlower = true;
            }
        }
    });

    // If no flower has been spawned, force spawn one at a random position
    if (!hasSpawnedFlower) {
        const randomPos = positions[Math.floor(Math.random() * positions.length)];
        const blockBelow = dimension.getBlock({ x: randomPos.x, y: randomPos.y - 1, z: randomPos.z });
        if (blockBelow && (blockBelow.typeId === "minecraft:grass_block" || blockBelow.typeId === "minecraft:dirt")) {
            dimension.runCommand(`setblock ${randomPos.x} ${randomPos.y} ${randomPos.z} ${flowerType}`);
        }
    }
}

// Function to drop loot based on block type
function dropLoot(block, dimension) {
    flowerTypes.forEach(type => {
        if ((block.typeId === type.bottom || block.typeId === type.top) && type.top) {
            const lootTable = `"blocks/${type.name}"`;
            dimension.runCommand(`loot spawn ${block.location.x} ${block.location.y} ${block.location.z} loot ${lootTable}`);
        }
    });
}

// Block components
const blockComponents = [
    {
        id: "biomes_and_beyond:flowerSpread",
        code: {
            onPlayerInteract: (data) => {
                const { block, dimension, player } = data;

                // Get the mainhand
                const mainhand = player.getComponent(EntityEquippableComponent.componentId).getEquipmentSlot(EquipmentSlot.Mainhand);
                // Get the held item
                const item = mainhand.getItem();
                // If there is no item, return
                if (!item) return;

                if (item.typeId === "minecraft:bone_meal") {
                    // Run this if the item identifier is bone meal

                    // Ensure block.typeId is defined before using includes
                    if (block.typeId) {
                        console.log(`Block Type ID: ${block.typeId}`);
                        
                        // Determine the flower type from the block ID
                        let flowerType = "";
                        flowerTypes.forEach(type => {
                            if (block.typeId.includes(type.bottom.split(":")[1]) || (type.top && block.typeId.includes(type.top.split(":")[1]))) {
                                flowerType = type.bottom; // Use the bottom block for spawning if applicable
                            }
                        });

                        console.log(`Flower Type: ${flowerType}`);

                        if (flowerType) {
                            const flowerTypeObject = flowerTypes.find(type => type.bottom === flowerType);

                            // If the flower can spread, handle spreading mechanics
                            if (flowerTypeObject && flowerTypeObject.spread) {
                                spawnFlowersAndGrassAround(block, dimension, flowerType);
                            }

                            // Drop loot only for tall flowers
                            if (flowerTypeObject && flowerTypeObject.top) {
                                dropLoot(block, dimension);
                            }

                            // Try to spawn the particle
                            try {
                                dimension.spawnParticle("minecraft:crop_growth_emitter", block.center());
                            } catch {}

                            // Play the sound
                            dimension.playSound("item.bone_meal.use", block.center());

                            // Return if the player's game mode is creative
                            if (player.getGameMode() === GameMode.creative) return;

                            // Set the amount to the item amount - 1
                            let amount = item.amount - 1;
                            if (amount > 0) {
                                // If the reduced amount is more than 0, set the item amount to the reduced amount
                                item.amount = amount;
                                mainhand.setItem(item);
                            } else {
                                // If the reduced amount isn't higher than 0, set the item to nothing
                                mainhand.setItem();
                            }
                        }
                    }
                }
            }
        }
    }
];

// Register custom components and set initial flower type
world.beforeEvents.worldInitialize.subscribe((data) => {
    // Register each block state
    for (const comp of blockComponents) {
        data.blockComponentRegistry.registerCustomComponent(comp.id, comp.code);
    }
});
