import { world, ItemStack, EquipmentSlot } from '@minecraft/server';

const SLAB_TYPES = [
    "fir",
    "jacaranda",
    "redwood",
    "baobab",
    "maple",
    "forsaken_oak",
    "chorus"
];

SLAB_TYPES.forEach(type => {
    // Destroy: Drop double slab as item
    world.beforeEvents.worldInitialize.subscribe(eventData => {
        eventData.blockComponentRegistry.registerCustomComponent(`biomes_and_beyond:${type}_slab_on_player_destroy`, {
            onPlayerDestroy(e) {
                const { player, dimension, destroyedBlockPermutation: perm } = e;
                if (!player) return;
                if (perm.getState('biomes_and_beyond:double')) {
                    const slabItem = perm.getItemStack(1);
                    if (slabItem) {
                        dimension.spawnItem(slabItem, e.block.location);
                    }
                }
            }
        });
    });

    // Interact: Place slab to make double
    world.beforeEvents.worldInitialize.subscribe(eventData => {
        eventData.blockComponentRegistry.registerCustomComponent(`biomes_and_beyond:${type}_slab_on_interact`, {
            onPlayerInteract(e) {
                const { block, player, face } = e;
                const equipment = player.getComponent('equippable');
                const selectedItem = equipment.getEquipment('Mainhand');
                if (selectedItem?.typeId === block.typeId && !block.permutation.getState('biomes_and_beyond:double')) {
                    const verticalHalf = block.permutation.getState('minecraft:vertical_half');
                    const isBottomUp = verticalHalf === 'bottom' && face === 'Up';
                    const isTopDown = verticalHalf === 'top' && face === 'Down';
                    if (isBottomUp || isTopDown) {
                        if (player.getGameMode() !== "creative") {
                            if (selectedItem.amount > 1) {
                                selectedItem.amount -= 1;
                                equipment.setEquipment('Mainhand', selectedItem);
                            } else {
                                equipment.setEquipment('Mainhand', undefined);
                            }
                        }
                        block.setPermutation(block.permutation.withState('biomes_and_beyond:double', true));
                        player.playSound('use.wood');
                    }
                }
            }
        });
    });
});