import { world, system, EquipmentSlot, ItemStack } from "@minecraft/server";

// The logs and their corresponding stripped types (including wood variants)
const LOG_TO_STRIPPED = {
    "biomes_and_beyond:fir_log": "biomes_and_beyond:stripped_fir_log",
    "biomes_and_beyond:baobab_log": "biomes_and_beyond:stripped_baobab_log",
    "biomes_and_beyond:forsaken_oak_log": "biomes_and_beyond:stripped_forsaken_oak_log",
    "biomes_and_beyond:jacaranda_log": "biomes_and_beyond:stripped_jacaranda_log",
    "biomes_and_beyond:redwood_log": "biomes_and_beyond:stripped_redwood_log",
    "biomes_and_beyond:chorus_log": "biomes_and_beyond:stripped_chorus_log",
    "biomes_and_beyond:maple_log": "biomes_and_beyond:stripped_maple_log",
    // Wood variants (stripped wood)
    "biomes_and_beyond:fir_wood": "biomes_and_beyond:stripped_fir_wood",
    "biomes_and_beyond:baobab_wood": "biomes_and_beyond:stripped_baobab_wood",
    "biomes_and_beyond:forsaken_oak_wood": "biomes_and_beyond:stripped_forsaken_oak_wood",
    "biomes_and_beyond:jacaranda_wood": "biomes_and_beyond:stripped_jacaranda_wood",
    "biomes_and_beyond:redwood_wood": "biomes_and_beyond:stripped_redwood_wood",
    "biomes_and_beyond:chorus_wood": "biomes_and_beyond:stripped_chorus_wood",
    "biomes_and_beyond:maple_wood": "biomes_and_beyond:stripped_maple_wood"
    // Add more logs and wood variants and their stripped types here as needed
};

world.beforeEvents.playerInteractWithBlock.subscribe((data) => {
    const Axes = [
        "minecraft:wooden_axe",
        "minecraft:stone_axe",
        "minecraft:gold_axe",
        "minecraft:iron_axe",
        "minecraft:diamond_axe",
        "minecraft:netherite_axe",
        "biomes_and_beyond:tungsten_axe",
        "biomes_and_beyond:enderium_axe"
    ];

    // Ensure the block and item stack are defined
    if (!data.block || !data.itemStack) return;

    // Check if the block is one of the logs or wood types that we want to strip
    const logType = data.block.typeId;
    const strippedType = LOG_TO_STRIPPED[logType];

    // Exit early if the block type is not in the LOG_TO_STRIPPED mapping
    if (!strippedType) return;

    // Only proceed if the item is an axe
    if (Axes.includes(data.itemStack.typeId)) {
        system.run(() => {
            try {
                // Get the direction of the block face (orientation)
                const Direction = data.block.permutation.getState("minecraft:block_face");

                // Set the block type to the stripped version
                data.block.setType(strippedType);
                // Preserve the block's orientation (block_face state)
                data.block.setPermutation(data.block.permutation.withState("minecraft:block_face", Direction));

                // Play the log stripping sound (Bedrock specific sound for stripping logs)
                const player = data.player;
                if (player) {
                    player.playSound("use.wood"); // Correct sound for log stripping in Bedrock
                }
            } catch (error) {
                console.error("Error processing log stripping:", error);
            }
        });
    }
});
