import { world, system, BlockPermutation, GameMode, EntityComponentTypes, EquipmentSlot } from '@minecraft/server';

function placeBlockAboveWater(source, blockPermutation, blockLocation) {
    const aboveBlockLocation = {
        x: blockLocation.x,
        y: blockLocation.y + 1,
        z: blockLocation.z
    };
    const blockAbove = source.dimension.getBlock(aboveBlockLocation);
    if (blockAbove) {
        blockAbove.setPermutation(blockPermutation);
    }
}

function decrementStack(player, quantity = 1) {
    if (!player.isValid() || player.getGameMode() === GameMode.creative) return;
    const equippableComp = player.getComponent(EntityComponentTypes.Equippable);
    const item = equippableComp.getEquipment(EquipmentSlot.Mainhand);
    if (!item) return;
    if (item.amount <= quantity) equippableComp.setEquipment(EquipmentSlot.Mainhand, null);
    else { item.amount -= quantity; equippableComp.setEquipment(EquipmentSlot.Mainhand, item); }
}

world.beforeEvents.itemUseOn.subscribe((data) => {
    const { source, block, itemStack } = data;
    if (!data.isFirstEvent || (block.typeId !== 'minecraft:water' && !block.isWaterlogged)) return;

    if (itemStack.typeId === 'biomes_and_beyond:lily_blossom_item') {
        data.cancel = true;
        system.run(() => {
            placeBlockAboveWater(source, BlockPermutation.resolve('biomes_and_beyond:lily_blossom'), block.location);
            decrementStack(source);
        });
    }

    if (itemStack.typeId === 'biomes_and_beyond:duckweed_item') {
        data.cancel = true;
        system.run(() => {
            placeBlockAboveWater(source, BlockPermutation.resolve('biomes_and_beyond:duckweed'), block.location);
            decrementStack(source);
        });
    }
});
