import { system, Player, ItemCustomComponent } from '@minecraft/server';

export class CoconutItem {
    constructor() {
        // Define the coconut item component
        this.coconutItemComponent = new ItemCustomComponent('biomes_and_beyond:coconut_item');
        this.coconutItemComponent.onCompleteUse = (eventData) => this.onCoconutThrown(eventData);
        
        // Add this item custom component to the system
        system.registerCustomComponent(this.coconutItemComponent);
    }

    // Handle when the coconut item is used (thrown)
    onCoconutThrown(eventData) {
        const player = eventData.player;
        const itemStack = eventData.itemStack;
        
        // Ensure it's the coconut item
        if (itemStack.__identifier__ === 'biomes_and_beyond:coconut_item') {
            // Deplete one coconut from the player's inventory
            this.depleteCoconut(player, itemStack);
        }
    }

    // Deplete one coconut from the player's inventory
    depleteCoconut(player, itemStack) {
        const inventory = player.getComponent('minecraft:inventory');
        
        // Find the coconut item in the inventory
        const coconutSlot = inventory.container.findFirst(itemStack.__identifier__);

        if (coconutSlot && coconutSlot.count > 0) {
            // Decrease the count of coconuts by 1
            coconutSlot.count -= 1;
        }

    }
}

// Initialize the CoconutItem component
const coconutItemScript = new CoconutItem();

// Run the script
coconutItemScript.initialize();
