import { BlockInventoryComponent, system, world } from '@minecraft/server';

class Raccoon {
    static IDENTIFIER = 'biomes_and_beyond:raccoon';
    static VALID_CONTAINERS = ['minecraft:chest'];
    static CHEST_RANGE = 8;
    static WAIT_TIME_RANGE = [1.0, 1.0];

    constructor() {
        this.registerStayDoneEvent();
    }

    registerStayDoneEvent() {
        system.afterEvents.scriptEventReceive.subscribe(({ id, sourceEntity }) => {
            if (id === "bab:stay_done" && sourceEntity?.typeId === Raccoon.IDENTIFIER) {
                this.searchChestAndStealItem(sourceEntity);
            }
        });
    }

    searchChestAndStealItem(raccoon) {
        if (!this.isMainHandEmpty(raccoon)) return;

        const chestBlock = this.getNearbyChestBlock(raccoon);
        if (chestBlock) {
            this.stealRandomItemFromChest(chestBlock, raccoon);
        }
    }

    isMainHandEmpty(raccoon) {
        try {
            raccoon.runCommand('testfor @s[hasitem={location=slot.weapon.mainhand}]');
            return false;
        } catch {
            return true;
        }
    }

    getNearbyChestBlock(entity) {
        const dimension = world.getDimension('overworld');
        const { x, y, z } = entity.location;
        const chestRange = Raccoon.CHEST_RANGE;

        for (let dx = -chestRange; dx <= chestRange; dx++) {
            for (let dz = -chestRange; dz <= chestRange; dz++) {
                const block = dimension.getBlock({ x: x + dx, y: y, z: z + dz });
                if (block && Raccoon.VALID_CONTAINERS.includes(block.typeId)) {
                    return block;
                }
            }
        }
        return null;
    }

    stealRandomItemFromChest(chestBlock, raccoon) {
        const inventoryComponent = chestBlock.getComponent(BlockInventoryComponent.componentId);
        if (!inventoryComponent) return;

        const inventory = inventoryComponent.container;
        const items = [];

        for (let i = 0; i < inventory.size; i++) {
            const item = inventory.getItem(i);
            if (item && this.hasFoodTag(item)) {
                items.push({ slot: i, item });
            }
        }

        if (items.length === 0) return;

        const waitTime = Math.floor(Math.random() * (Raccoon.WAIT_TIME_RANGE[1] - Raccoon.WAIT_TIME_RANGE[0] + 1)) + Raccoon.WAIT_TIME_RANGE[0];

        // Play chest opening sound when the raccoon interacts with the chest
        world.playSound("random.chestopen", chestBlock.location);

        system.runTimeout(() => {
            const slot = Math.floor(Math.random() * items.length);
            const selectedItem = items[slot];

            const newAmount = selectedItem.item.amount - 1;
            if (newAmount === 0) {
                inventory.setItem(selectedItem.slot, null);
            } else {
                selectedItem.item.amount = newAmount;
                inventory.setItem(selectedItem.slot, selectedItem.item);
            }

            raccoon.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 ${selectedItem.item.typeId}`);

            // Play chest closing sound after stealing
            world.playSound("random.chestclosed", chestBlock.location);

        }, 20 * waitTime);
    }

    hasFoodTag(item) {
        return item.hasTag("minecraft:is_food");
    }
}

new Raccoon();
