import { system, world, BlockPermutation, Direction, EntityDamageCause, EquipmentSlot, GameMode, ItemStack } from "@minecraft/server";

// --- default_world_arrays.js ---
const addonNamespaces = ["biomes_and_beyond:"];
const isAxeTagInteractBlocks = [
    {
        matches: id=>id.includes("_log")||id.includes("_wood"),
        sound: id=>id.includes("cherry")?"step.cherry_wood":"use.wood",
        pitch: 0.8
    },
    {
        matches: id=>id.includes("_stem")||id.includes("_hyphae"),
        sound: "use.stem",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:stripped_bamboo_block",
        sound: "step.bamboo_wood",
        pitch: 0.8
    }
];
const isHoeTagInteractBlocks = [
    {
        matches: (id,block)=>id=="minecraft:farmland"||block.hasTag("grass"),
        sound: "use.gravel",
        pitch: 0.8
    }
];
const isShovelTagInteractBlocks = [
    {
        matches: id=>id=="minecraft:grass_path",
        sound: "use.grass",
        pitch: 0.8
    }
];
const isShovelTagUseOnBlocks = [
    {
        matches: id=>id=="minecraft:coarse_dirt",
        sound: "use.grass",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:dirt",
        sound: "use.grass",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:dirt_with_roots",
        sound: "use.grass",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:mycelium",
        sound: "use.grass",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:podzol",
        sound: "use.grass",
        pitch: 0.8
    },
    {
        matches: id=>id=="minecraft:campfire"||id=="minecraft:soul_campfire",
        sound: "random.fizz",
        pitch: ()=>1.8+Math.random()*0.6,
        volume: 0.5
    }
];
const isShovelTagUseOnBlocksToGrassPath = [
    "minecraft:coarse_dirt",
    "minecraft:dirt",
    "minecraft:dirt_with_roots",
    "minecraft:mycelium",
    "minecraft:podzol"
];
const isShovelTagUseOnBlocksToExtinguish = [
    "minecraft:campfire",
    "minecraft:soul_campfire"
];

// --- world_utilities.js ---
function interactDamageTool(durability,enchantable,mainhand,player,itemStack) {
    if (player.getGameMode()==GameMode.creative) return;
    const unbreaking = enchantable?.getEnchantment("unbreaking")?.level;
    const damageChance = 1/(unbreaking+1);
    if (Math.random()>damageChance) return;
    if (durability.damage>=durability.maxDurability) {
        mainhand.setItem(undefined);
        player.playSound("random.break");
    }
    else {
        durability.damage++;
        mainhand.setItem(itemStack);
    }
}

function interactToolUse(block,itemStack,player,interactBlocks) {
    const mainhand = player.getComponent("minecraft:equippable").getEquipmentSlot(EquipmentSlot.Mainhand);
    const durability = itemStack.getComponent("minecraft:durability");
    const enchantable = itemStack.getComponent("minecraft:enchantable");
    const blockId = block.typeId;
    const center = block.center();
    for (const{matches,sound="use.stone",pitch=1.0,volume=1.0} of interactBlocks) {
        if (matches(blockId,block)) {
            interactDamageTool(durability,enchantable,mainhand,player,itemStack);
            const soundId = typeof sound=="function"?sound(blockId):sound;
            const pitchRange = typeof pitch=="function"?pitch():pitch;
            const volumeRange = typeof volume=="function"?volume():volume;
            player.playSound(soundId,{location:center,pitch:pitchRange,volume:volumeRange});
            return true;
        }
    }
    return false;
}

// --- world_modules.js main ---
world.beforeEvents.itemUseOn.subscribe(({source:player,block,itemStack}) => {
    const blockAbove = block.dimension.getBlock(block.location).above(1);
    const id = itemStack.typeId;
    if (addonNamespaces.some(prefix=>id.startsWith(prefix))) {
        if (itemStack.hasTag("minecraft:is_shovel")) {
            if (isShovelTagUseOnBlocksToGrassPath.includes(block.typeId)) {
                if (!blockAbove.isAir) return;
                system.run(() => {
                    interactToolUse(block,itemStack,player,isShovelTagUseOnBlocks);
                    return block.setType("minecraft:grass_path");
                });
            }
            if (isShovelTagUseOnBlocksToExtinguish.includes(block.typeId)) {
                const extinguished = block.permutation.getState("extinguished");
                if (extinguished==true) return;
                system.run(() => {
                    interactToolUse(block,itemStack,player,isShovelTagUseOnBlocks);
                    return block.setPermutation(block.permutation.withState("extinguished",true));
                });
            }
        }
    }
});
world.afterEvents.playerInteractWithBlock.subscribe(({block,itemStack,player}) => {
    if (!itemStack) return;
    const id = itemStack.typeId;
    if (addonNamespaces.some(prefix=>id.startsWith(prefix))) {
        if (itemStack.hasTag("minecraft:is_axe")) return interactToolUse(block,itemStack,player,isAxeTagInteractBlocks);
        if (itemStack.hasTag("minecraft:is_hoe")) return interactToolUse(block,itemStack,player,isHoeTagInteractBlocks);
        if (itemStack.hasTag("minecraft:is_shovel")) return interactToolUse(block,itemStack,player,isShovelTagInteractBlocks);
    }
});