import { world, system } from '@minecraft/server';

// Define the function to perform when the entity is within the radius
function performActionWhenNearby(player) {
    try {
        player.runCommandAsync('function sandstorm_fog');
    } catch (error) {
        console.warn(`Failed to execute command for ${player.name}: ${error}`);
    }
}

// Define the function to perform when the entity is no longer within the radius
function performActionWhenFar(player) {
    try {
        player.runCommandAsync('function sandstorm_fog_remove');
    } catch (error) {
        console.warn(`Failed to execute command for ${player.name}: ${error}`);
    }
}

// Subscribe to the tick event
system.runInterval(() => {
    const players = world.getPlayers();
    const radius = 64;

    for (const player of players) {
        let entityNearby = false;
        const playerPos = player.location;

        // Check for entities within the radius
        const entities = world.getDimension('overworld').getEntities({ type: 'biomes_and_beyond:weather_detect' });
        for (const entity of entities) {
            const entityPos = entity.location;
            const distance = Math.sqrt(
                Math.pow(entityPos.x - playerPos.x, 2) +
                Math.pow(entityPos.y - playerPos.y, 2) +
                Math.pow(entityPos.z - playerPos.z, 2)
            );

            if (distance <= radius) {
                entityNearby = true;
                break;
            }
        }

        // Perform actions based on the presence of the entity
        if (entityNearby) {
            if (!player.hasTag('weather_detect_nearby')) {
                performActionWhenNearby(player);
                player.addTag('weather_detect_nearby');
            }
        } else {
            if (player.hasTag('weather_detect_nearby')) {
                performActionWhenFar(player);
                player.removeTag('weather_detect_nearby');
            }
        }
    }
}, 100); // Run the interval every 100 ticks (5 seconds)
